Option Strict Off
Option Explicit On
Imports VB = Microsoft.VisualBasic
Friend Class clsSun
	
	'Copyright (C) 2004 Kristian. S.Stangeland


	'This program is free software; you can redistribute it and/or
	'modify it under the terms of the GNU General Public License
	'as published by the Free Software Foundation; either version 2
	'of the License, or (at your option) any later version.
	
	'This program is distributed in the hope that it will be useful,
	'but WITHOUT ANY WARRANTY; without even the implied warranty of
	'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	'GNU General Public License for more details.
	
	'You should have received a copy of the GNU General Public License
	'along with this program; if not, write to the Free Software
	'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	
	Const pi As Double = 3.14159265358979
	Const RADEG As Double = 180 / pi
	Const DEGRAD As Double = pi / 180
	
	Dim lYear As Integer
	Dim lMonth As Integer
	Dim lDay As Integer
	
	Dim lSunSet As Date
	Dim lSunRise As Date
	Dim lChanged As Boolean
	Dim lLongitude As Double
	Dim lLatitude As Double
	Dim lTZ As Double
	Dim lIsDst As Double
	
	Public ReadOnly Property Name() As String
		Get
			
			Name = "SunTime"
			
		End Get
	End Property
	
	Public ReadOnly Property SunSet() As Date
		Get
			
            SunSet = System.DateTime.FromOADate(lSunSet.ToOADate + (IIf(Sommertid, IIf(DaylightSaving(DateSerial(lYear, lMonth, lDay)), TimeSerial(1, 0, 0), 0), 0)).ToOADate)
			
		End Get
	End Property
	
	Public ReadOnly Property SunRise() As Date
		Get
			
			SunRise = System.Date.FromOADate(lSunRise.ToOADate + (IIf(Sommertid, IIf(DaylightSaving(DateSerial(lYear, lMonth, lDay)), TimeSerial(1, 0, 0), 0), 0)).ToOADate)
			
		End Get
	End Property
	
	Public ReadOnly Property Changed() As Integer
		Get
			
			Changed = lChanged
			
		End Get
	End Property
	
	
	Public Property cYear() As Integer
		Get
			
			cYear = lYear
			
		End Get
		Set(ByVal Value As Integer)
			
			lChanged = lChanged Or CBool(lYear <> Value)
			lYear = Value
			
		End Set
	End Property
	
	
	Public Property cMonth() As Integer
		Get
			
			cMonth = lMonth
			
		End Get
		Set(ByVal Value As Integer)
			
			lChanged = lChanged Or CBool(lMonth <> Value)
			lMonth = Value
			
		End Set
	End Property
	
	
	Public Property cDay() As Integer
		Get
			
			cDay = lDay
			
		End Get
		Set(ByVal Value As Integer)
			
			lChanged = lChanged Or CBool(lDay <> Value)
			lDay = Value
			
		End Set
	End Property
	
	
	Public Property cCurrentDate() As Date
		Get
			
			cCurrentDate = DateSerial(lYear, lMonth, lDay)
			
		End Get
		Set(ByVal Value As Date)
			
			cYear = Year(Value)
			cMonth = Month(Value)
			cDay = VB.Day(Value)
			
		End Set
	End Property
	
	
	Public Property Longitude() As Double
		Get
			
			Longitude = lLongitude
			
		End Get
		Set(ByVal Value As Double)
			
			lChanged = lChanged Or CBool(lLongitude <> Value)
			lLongitude = Value
			
		End Set
	End Property
	
	
	Public Property Latitude() As Double
		Get
			
			Latitude = lLatitude
			
		End Get
		Set(ByVal Value As Double)
			
			lChanged = lChanged Or CBool(lLatitude <> Value)
			lLatitude = Value
			
		End Set
	End Property
	
	
	Public Property TimeZone() As Double
		Get
			
			TimeZone = lTZ
			
		End Get
		Set(ByVal Value As Double)
			
			lChanged = lChanged Or CBool(lTZ <> Value)
			lTZ = Value
			
		End Set
	End Property
	
	
	Public Property Sommertid() As Double
		Get
			
			Sommertid = lIsDst
			
		End Get
		Set(ByVal Value As Double)
			
			lChanged = lChanged Or CBool(lIsDst <> Value)
			lIsDst = Value
			
		End Set
	End Property
	
	Public ReadOnly Property cLeapYear() As Integer
		Get
			
			If (cYear Mod 4 = 0) And (cYear Mod 100 <> 0) Or (cYear Mod 400 = 0) Then
				cLeapYear = 29
			Else
				cLeapYear = 28
			End If
			
		End Get
	End Property
	
	Public Sub Calculate()
		
		Dim ye, ecl, YS, V, xv, e1, L, w, n, d, i, m, E, a, yv, XS, xe, lonsun, ze As Double
		Dim min_rise, hour_rise, dec, r, UT_Sun_in_south, GMST0, LHA, ra, h, hour_set, min_set As Double
		
		d = (367 * (lYear) - Int((7 * ((lYear) + (((lMonth) + 9) / 12))) / 4) + Int((275 * (lMonth)) / 9) + (lDay) - 730530)
		
		' Orbital elements of the Sun:
		n = 0#
		i = 0#
		w = 282.9404 + 0.0000470935 * d
		
		a = 1#
		E = 0.016709 - 0.000000001151 * d
		
		m = 356.047 + 0.9856002585 * d
		'UPGRADE_WARNING: Couldn't resolve default property of object Rev(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		m = Rev(m)
		
		ecl = 23.4393 - 0.0000003563 * d
		L = w + m
		
		If (L < 0 Or L > 360) Then
			'UPGRADE_WARNING: Couldn't resolve default property of object Rev(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
			L = Rev(L)
		End If
		
		' position of the Sun
		'UPGRADE_WARNING: Couldn't resolve default property of object Cosd(m). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		e1 = m + E * (180 / pi) * Sind(m) * (1# + E * Cosd(m))
		'UPGRADE_WARNING: Couldn't resolve default property of object Cosd(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		xv = Cosd(e1) - E
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(e1). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Sqrt(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		yv = Sqrt(1# - E * E) * Sind(e1)
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Atan2d(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		V = Atan2d(yv, xv)
		'UPGRADE_WARNING: Couldn't resolve default property of object Sqrt(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		r = Sqrt(xv * xv + yv * yv)
		lonsun = V + w
		
		If (lonsun < 0 Or lonsun > 360) Then
			'UPGRADE_WARNING: Couldn't resolve default property of object Rev(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
			lonsun = Rev(lonsun)
		End If
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Cosd(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		XS = r * Cosd(lonsun)
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		YS = r * Sind(lonsun)
		xe = XS
		'UPGRADE_WARNING: Couldn't resolve default property of object Cosd(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		ye = YS * Cosd(ecl)
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		ze = YS * Sind(ecl)
		'UPGRADE_WARNING: Couldn't resolve default property of object Atan2d(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		ra = Atan2d(ye, xe)
		'UPGRADE_WARNING: Couldn't resolve default property of object Sqrt(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Atan2d(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		dec = Atan2d(ze, Sqrt((xe * xe) + (ye * ye)))
		h = -0.833
		
		GMST0 = L + 180
		
		If (GMST0 < 0 Or GMST0 > 360) Then
			'UPGRADE_WARNING: Couldn't resolve default property of object Rev(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
			GMST0 = Rev(GMST0)
		End If
		
		UT_Sun_in_south = (ra - GMST0 - lLongitude) / 15#
		
		If (UT_Sun_in_south < 0) Then
			UT_Sun_in_south = UT_Sun_in_south + 24
		End If
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Cosd(dec). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Cosd(lLatitude). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(dec). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(lLatitude). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Sind(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		LHA = (Sind(h) - (Sind(lLatitude) * Sind(dec))) / (Cosd(lLatitude) * Cosd(dec))
		'UPGRADE_WARNING: Couldn't resolve default property of object Acosd(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		LHA = Acosd(LHA) / 15
		
		hour_rise = UT_Sun_in_south - LHA
		hour_set = UT_Sun_in_south + LHA
		min_rise = Int((hour_rise - Int(hour_rise)) * 60)
		min_set = Int((hour_set - Int(hour_set)) * 60)
		
		hour_rise = (Int(hour_rise) + (lTZ + lIsDst))
		hour_set = (Int(hour_set) + (lTZ + lIsDst))
		
		lChanged = False
		lSunRise = TimeSerial(hour_rise, min_rise, 0)
		lSunSet = TimeSerial(hour_set, min_set, 0)
		
	End Sub
	
	' Support Functions
	Private Function Sind(ByRef qqq As Double) As Object
		
		Sind = System.Math.Sin((qqq) * DEGRAD)
		
	End Function
	
	Private Function Cosd(ByRef qqq As Double) As Object
		
		Cosd = System.Math.Cos((qqq) * DEGRAD)
		
	End Function
	
	Private Function Tand(ByRef qqq As Double) As Object
		
		Tand = System.Math.Tan((qqq) * DEGRAD)
		
	End Function
	
	Private Function Acosd(ByRef qqq As Double) As Object
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Acos(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Acosd. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		Acosd = (RADEG * Acos(qqq))
		
	End Function
	
	Private Function Atan2d(ByRef qqq As Double, ByRef qqq1 As Double) As Object
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Atan2(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Atan2d. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		Atan2d = (RADEG * Atan2(qqq, qqq1))
		
	End Function
	
	Private Function Rev(ByRef qqq As Double) As Object
		
		Dim X As Double
		
		X = (qqq - Int(qqq / 360#) * 360#)
		
		If (X <= 0) Then
			X = X + 360
		End If
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Rev. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		Rev = X
		
	End Function
	
	Private Function Atan2(ByRef YS As Double, ByRef XS As Double) As Object
		
		Dim Theta As Double
		
		If XS <> 0 Then
			
			Theta = System.Math.Atan(YS / XS)
			
			If XS < 0 Then
				Theta = Theta + pi
			End If
		Else
			
			If YS < 0 Then
				Theta = 3 * pi / 2 '90
			Else
				Theta = pi / 2 '270
			End If
			
		End If
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Atan2. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		Atan2 = Theta
		
	End Function
	
	Private Function Acos(ByRef X As Double) As Object
		
		'UPGRADE_WARNING: Couldn't resolve default property of object Sqrt(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		'UPGRADE_WARNING: Couldn't resolve default property of object Acos. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
		Acos = System.Math.Atan(-X / Sqrt(-X * X + 1)) + 2 * System.Math.Atan(1)
		
	End Function
	
	Private Function Sqrt(ByRef X As Double) As Object
		
		If X > 0 Then
			Sqrt = System.Math.Sqrt(X)
		Else
			'UPGRADE_WARNING: Couldn't resolve default property of object Sqrt. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
			Sqrt = 0
		End If
		
	End Function
	
	Private Function LastWeekday(ByRef lWeekday As Integer, ByRef lMonth As Integer, Optional ByRef dayNum As Integer = 1) As Date
		
		Dim lDate As Date
		Dim Tell As Integer
		
		For Tell = GetMonthLenght(lMonth) To 1 Step -1
			
			lDate = DateSerial(cYear, lMonth, Tell)
			
			If WeekDay(lDate) = lWeekday Then
				
				dayNum = dayNum - 1
				
				If dayNum <= 0 Then
					LastWeekday = lDate
					Exit Function
				End If
				
			End If
			
		Next 
		
	End Function
	
	'UPGRADE_NOTE: Month was upgraded to Month_Renamed. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="A9E4979A-37FA-4718-9994-97DD76ED70A7"'
	Private Function GetMonthLenght(ByRef Month_Renamed As Integer) As Integer
		
		Dim Tmp As Integer
		
		Select Case Month_Renamed
			Case 1 : Tmp = 31
			Case 2 : Tmp = cLeapYear
			Case 3 : Tmp = 31
			Case 4 : Tmp = 30
			Case 5 : Tmp = 31
			Case 6 : Tmp = 30
			Case 7 : Tmp = 31
			Case 8 : Tmp = 31
			Case 9 : Tmp = 30
			Case 10 : Tmp = 31
			Case 11 : Tmp = 30
			Case 12 : Tmp = 31
		End Select
		
		GetMonthLenght = Tmp
		
	End Function
	
	Private Function DaylightSaving(ByRef CurrDate As Date) As Boolean
		
		DaylightSaving = CBool(CurrDate >= LastWeekday(FirstDayOfWeek.Sunday, 3) And CurrDate <= LastWeekday(FirstDayOfWeek.Sunday, 10))
		
	End Function
	
	'UPGRADE_NOTE: Class_Initialize was upgraded to Class_Initialize_Renamed. Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="A9E4979A-37FA-4718-9994-97DD76ED70A7"'
	Private Sub Class_Initialize_Renamed()
		
		cCurrentDate = Today
		
	End Sub
	Public Sub New()
		MyBase.New()
		Class_Initialize_Renamed()
	End Sub
End Class